﻿XIncludeFile "InputStream.pbi"

; See: http://docs.oracle.com/javase/7/docs/api/java/io/ByteArrayInputStream.html
Class(MemoryInputStream, InputStream)
		; does nothing
		;close.i()
		
		; bytes available for reading
		;available.q()
		
		; reads size byes from the stream into *buffer.
		; returns bytes copied or -1 if end of stream was reached.
		;readBuffer.i(*buffer, size.i)
		
		; skips a certain amount of bytes.
		;skip.q(bytes.q)
		
		; marks a position in the stream to which can be jumped with reset
		; initially 0. readLimit is ignored.
		;mark(readLimit.q)
		
		; returns true
		;markSupported.i()
		
		; jumps to the marked position.
		;reset()
	Members
		*buffer
		copied.i
		size.i
		pos.i
		marked.i
	Constructors
		Construct(createFrom,(*buffer, size.i))
		Construct(copyFrom,(*buffer, size.i))
	Definition
		PrivateDefaultConstructor
			this\pos = 0
			this\marked = 0
			this\copied = #False
		EndDefaultConstructor
		
		Constructor(createFrom,(*buffer, size.i))
			CallConstructor()
			this\buffer = *buffer
			this\size = size
		EndConstructor
		
		Constructor(copyFrom,(*buffer, size.i))
			Protected *tbuffer = AllocateMemory(size)
			If Not *tbuffer
				ConstructorFailure
			EndIf
			CopyMemory(*buffer, *tbuffer, size)
			CallConstructor(createFrom,(*tbuffer,size))
			this\copied = #True
		EndConstructor
		
		Deconstructor
			If this\copied
				FreeMemory(this\buffer)
			EndIf
		EndDeconstructor
		
		Synchronized(i, close,(this))
			;does nothing
		EndSynchronized
		
		Synchronized(q, available,(this))
			SynchronizedReturn(this\size - this\pos)
		EndSynchronized
		
		Synchronized(i, readBuffer,(this, *buffer, size.i))
			If this\pos = this\size
				SynchronizedReturn(-1)
			ElseIf this\size - this\pos < size
				size = this\size - this\pos
			EndIf
			
			CopyMemory(this\buffer + this\pos, *buffer, size)
			this\pos + size
			
			SynchronizedReturn(size)
		EndSynchronized
		
		Synchronized(q, skip,(this, bytes.q))
			If this\size - this\pos < bytes
				bytes = this\size - this\pos
			EndIf
			this\pos + bytes
			
			SynchronizedReturn(bytes)
		EndSynchronized
		
		Synchronized(i, mark,(this, readLimit.q))
			this\marked = this\pos
		EndSynchronized
		
		Synchronized(i, markSupported,(this))
			SynchronizedReturn(#True)
		EndSynchronized
		
		Synchronized(i, reset,(this))
			this\pos = this\marked
		EndSynchronized
EndClass

CompilerIf #PB_Compiler_IsMainFile
	Define bufferString.s = "%Garbage%This is a test  for the MemoryInputStream class."
	
	Define *buffer = AllocateMemory(Len(bufferString))
	PokeS(*buffer, bufferString, -1, #PB_Ascii | #PB_String_NoZero)
	
	Define is.type(MemoryInputStream) = MemoryInputStream::copyFrom(*buffer, Len(bufferString))
	
	; because copyFrom we can free or buffer
	FreeMemory(*buffer)
	
	Debug "Bytes available: " + is\available()
	Debug "Skip 9 bytes..."
	is\skip(9)
	Debug "Bytes available: " + is\available()
	Debug "Output word by word..."
	
	Repeat
		; Mark current position in stream
		is\mark(0)
		
		; Read bytes until a whitespace appears
		Define wordLength.q = 0
		Repeat
			Define c.a, bytesRead.q
			; Be sure to read one byte
			Repeat
				bytesRead = is\readBuffer(@c, 1)
			Until bytesRead <> 0
			wordLength + Bool(c <> ' ')
		Until c = ' ' Or bytesRead = -1
		
		; If word length is zero do not allocate a buffer
		If wordLength = 0
			Debug ""
			Continue
		EndIf
		
		; Reset position instream
		is\reset()
		
		; allocate memory for whole word
		*buffer = AllocateMemory(wordLength)
		
		; read word into buffer
		is\readBuffer(*buffer, wordLength)
		
		; skip whitespace
		is\skip(1)
		
		; output word
		Debug PeekS(*buffer, wordLength, #PB_Ascii)
		
		FreeMemory(*buffer)
		
	Until bytesRead = -1
	
	is\free()
	
CompilerEndIf
; IDE Options = PureBasic 5.40 LTS Beta 9 (Linux - x64)
; CursorPosition = 113
; FirstLine = 96
; Folding = -
; EnableUnicode
; EnableXP
; CompileSourceDirectory
; EnablePurifier